@@ -22,7 +22,7 @@ class IsosurfaceCollection(FDSDataCollection):
         if type(quantity) != str:
             quantity = quantity.name
         return IsosurfaceCollection(x for x in self if
-                                    x.name.name.lower() == quantity.lower() or x.name.label.lower() == quantity.lower())
+                                    x.quantity.name.lower() == quantity.lower() or x.name.label.lower() == quantity.lower())
 
     def __repr__(self):
         return "IsosurfaceCollection(" + super(IsosurfaceCollection, self).__repr__() + ")"
